Resource

Struct Resource 

pub struct Resource<RA: ResourceAlgebra> { /* private fields */ }
Interface for Resource Algebra ghost state.

Implementations

Source

impl<RA: ResourceAlgebra> Resource<RA>

pub proof fn as_ref(tracked &self) -> tracked r : ResourceRef<'_, RA>

ensures
self.loc() == r.loc(),
self.value() == r.value(),
Source

impl<RA: ResourceAlgebra> Resource<RA>

Implementation of a Resource based on a ResourceAlgebra

pub closed spec fn value(self) -> RA

The underlying value of the resource

pub closed spec fn loc(self) -> Loc

The location of the resource

pub proof fn alloc(value: RA) -> tracked out : Self

requires
value.valid(),
ensures
out.value() == value,

Allocate a new resource at a fresh location

pub proof fn join(tracked self, tracked other: Self) -> tracked out : Self

requires
self.loc() == other.loc(),
ensures
out.loc() == self.loc(),
out.value() == RA::op(self.value(), other.value()),

We can join together two resources at the same location, where we obtain the combination of the two

pub proof fn split(tracked self, left: RA, right: RA) -> tracked out : (Self, Self)

requires
self.value() == RA::op(left, right),
ensures
out.0.loc() == self.loc(),
out.1.loc() == self.loc(),
out.0.value() == left,
out.1.value() == right,

If a resource holds the result of a composition, we can split it into the components

pub proof fn validate(tracked &self)

ensures
self.value().valid(),

Whenever we have a resource, that resource is valid. This intuitively (and inductively) holds because: 1. We only create valid resources (via alloc); 2. We can only update the value of a resource via a frame_preserving_update_opt (see update for more details. Because of the requirement of that updates are frame preserving this means that joins remain valid.

pub proof fn update_nondeterministic(tracked self, new_values: Set<RA>) -> tracked out : Self

requires
frame_preserving_update_nondeterministic_opt(self.value(), new_values),
ensures
out.loc() == self.loc(),
new_values.contains(out.value()),

This is a more general version of update.

pub proof fn update(tracked self, new_value: RA) -> tracked out : Self

requires
frame_preserving_update_opt(self.value(), new_value),
ensures
out.loc() == self.loc(),
out.value() == new_value,

Update a resource to a new value. This can only be done if the update is frame preserving (i.e., any value that could be composed with the original value can still be composed with the new value).

See frame_preserving_update for more information.

pub proof fn extract(tracked &mut self) -> tracked other : Self

ensures
other.loc() == old(self).loc(),
other.value() == old(self).value(),

Extracts the resource from r, leaving r unspecified and returning a new resource holding the previous value of r.

pub proof fn validate_2(tracked &mut self, tracked other: &ResourceRef<'_, RA>)

requires
old(self).loc() == other.loc(),
ensures
*self == *old(self),
RA::op(self.value(), other.value()).valid(),

Validate two items at once. If we have two resources at the same location then its composition is valid.

pub proof fn update_nondeterministic_with_shared(tracked self, tracked other: &ResourceRef<'_, RA>, new_values: Set<RA>, ) -> tracked out : Self

requires
self.loc() == other.loc(),
frame_preserving_update_nondeterministic_opt(
    RA::op(self.value(), other.value()),
    set_op(new_values, other.value()),
),
ensures
out.loc() == self.loc(),
new_values.contains(out.value()),

We can do a similar update to update_with_shared for non-deterministic updates

pub proof fn update_with_shared(tracked self, tracked other: &ResourceRef<'_, RA>, new_value: RA, ) -> tracked out : Self

requires
self.loc() == other.loc(),
frame_preserving_update_opt(
    RA::op(self.value(), other.value()),
    RA::op(new_value, other.value()),
),
ensures
out.loc() == self.loc(),
out.value() == new_value,

If x · y --> x · z is a frame-perserving update, and we have a shared reference to x, we can update the y resource to z.