ResourceRef

Struct ResourceRef 

pub struct ResourceRef<'a, RA: ResourceAlgebra> { /* private fields */ }

Implementations

Source

impl<'a, RA: ResourceAlgebra> ResourceRef<'a, RA>

pub closed spec fn loc(self) -> Loc

pub closed spec fn value(self) -> RA

pub proof fn validate(tracked &self)

ensures
self.value().valid(),

Whenever we have a resource, that resource is valid. See Resource::validate for more details

pub proof fn join_shared(tracked &self, tracked other: &Self) -> tracked out : Self

requires
self.loc() == other.loc(),
ensures
out.loc() == self.loc(),
incl(self.value(), out.value()) || self.value() == out.value(),
incl(other.value(), out.value()) || other.value() == out.value(),

This is useful when you have two (or more) shared resources and want to learn that they agree, as you can combine this validate, e.g., x.join_shared(y).validate().

pub proof fn weaken(tracked &self, target: RA) -> tracked out : ResourceRef<'a, RA>

requires
incl(target, self.value()),
ensures
out.loc() == self.loc(),
out.value() == target,

Extract a shared reference to a preceding element in the extension order from a shared reference.

pub proof fn duplicate_previous(tracked &self, value: RA) -> tracked out : Resource<RA>

requires
frame_preserving_update_opt(self.value(), RA::op(value, self.value())),
ensures
out.loc() == self.loc(),
out.value() == value,

If x --> x · y is a frame-perserving update, and we have a shared reference to x, we can create a resource to y

pub proof fn join_shared_to_target(tracked &self, tracked other: &Self, target: RA) -> tracked out : Self

requires
self.loc() == other.loc(),
conjunct_shared(Some(self.value()), Some(other.value()), Some(target)),
ensures
out.loc() == self.loc(),
out.value() == target,

We can do a similar update to update_with_shared for non-deterministic updates